<template>
  <div class="p-2">
    <transition :enter-active-class="proxy?.animate.searchAnimate.enter" :leave-active-class="proxy?.animate.searchAnimate.leave">
      <div v-show="showSearch" class="mb-[10px]">
        <el-card shadow="hover">
          <el-form ref="queryFormRef" :model="queryParams" :inline="true">
            <el-form-item label="产品型号" prop="prodModel">
              <el-select v-model="queryParams.prodModel" placeholder="请选择产品型号" clearable @keyup.enter="handleQuery">
                <el-option v-for="item in prodModelOptions" :key="item.value" :label="item.label" :value="item.value" />
              </el-select>
            </el-form-item>
            <el-form-item label="测试项目" prop="testItem">
              <el-select v-model="selectedTestItems" :multiple="false" placeholder="请选择测试项目" clearable @keyup.enter="handleQuery">
                <el-option v-for="item in testItemsOptions" :key="item.value" :label="item.label" :value="item.value" />
              </el-select>
            </el-form-item>
            <el-form-item label="批次号" prop="batchCode">
              <el-input v-model="queryParams.batchCode" placeholder="请输入批次号" clearable @keyup.enter="handleQuery" />
            </el-form-item>
            <el-form-item label="创建时间" style="width: 308px">
              <el-date-picker
                v-model="dateRangeCreateTime"
                value-format="YYYY-MM-DD"
                type="daterange"
                range-separator="-"
                start-placeholder="开始日期"
                end-placeholder="结束日期"
                unlink-panels
                :default-time="[new Date(2000, 1, 1, 0, 0, 0), new Date(2000, 1, 1, 23, 59, 59)]"
              />
            </el-form-item>
            <el-form-item>
              <el-button type="primary" icon="Search" @click="handleQuery">搜索</el-button>
              <el-button icon="Refresh" @click="resetQuery">重置</el-button>
            </el-form-item>
          </el-form>
        </el-card>
      </div>
    </transition>

    <el-card shadow="never">
      <template #header>
        <el-row :gutter="10" class="mb8">
          <el-col :span="1.5">
            <el-button type="primary" plain icon="Plus" @click="handleAdd" v-hasPermi="['qms:testResult:add']">新增</el-button>
          </el-col>
          <el-col :span="1.5">
            <el-button type="success" plain icon="Edit" :disabled="single" @click="handleUpdate()" v-hasPermi="['qms:testResult:edit']">修改</el-button>
          </el-col>
          <el-col :span="1.5">
            <el-button type="danger" plain icon="Delete" :disabled="multiple" @click="handleDelete()" v-hasPermi="['qms:testResult:remove']">删除</el-button>
          </el-col>
          <el-col :span="1.5">
            <el-button type="warning" plain icon="Download" @click="handleExport" v-hasPermi="['qms:testResult:export']">导出</el-button>
          </el-col>
          <right-toolbar v-model:showSearch="showSearch" @queryTable="handleQuery"></right-toolbar>
        </el-row>
      </template>


      <div ref="chartRef" style="width: 100%; height: 660px;"></div>

      <!--      <pagination v-show="total > 0" :total="total" v-model:page="queryParams.pageNum" v-model:limit="queryParams.pageSize" @pagination="getList" />-->
    </el-card>
    <!-- 添加或修改测试结果对话框 -->
    <el-dialog :title="dialog.title" v-model="dialog.visible" width="500px" append-to-body>
      <el-form ref="testResultFormRef" :model="form" :rules="rules" label-width="80px">
        <el-form-item label="批次号" prop="batchCode">
          <el-input v-model="form.batchCode" placeholder="请输入批次号" />
        </el-form-item>
        <el-form-item label="测试序号" prop="testNum">
          <el-input v-model="form.testNum" placeholder="请输入测试序号" />
        </el-form-item>
        <el-form-item label="测试项目" prop="testItem">
          <el-select v-model="form.testItem" placeholder="请选择测试项目">
            <el-option v-for="item in testItemsOptions" :key="item.value" :label="item.label" :value="item.value" />
          </el-select>
        </el-form-item>
        <el-form-item label="实际电压" prop="voltage">
          <el-input v-model="form.voltage" placeholder="请输入实际电压" />
        </el-form-item>
        <el-form-item label="实际电流" prop="loadCurrent">
          <el-input v-model="form.loadCurrent" placeholder="请输入实际电流" />
        </el-form-item>
        <el-form-item label="标准距离" prop="stdDistance">
          <el-input v-model="form.stdDistance" placeholder="请输入标准距离" />
        </el-form-item>
        <el-form-item label="感应物" prop="inductor">
          <el-input v-model="form.inductor" placeholder="请输入感应物" />
        </el-form-item>
        <el-form-item label="输出引脚" prop="output">
          <el-input v-model="form.output" placeholder="请输入输出引脚" />
        </el-form-item>
        <el-form-item label="测试数据" prop="testValue">
          <el-input v-model="form.testValue" placeholder="请输入测试数据" />
        </el-form-item>
        <el-form-item label="判断条件" prop="judgeDetail">
          <el-input v-model="form.judgeDetail" placeholder="请输入判断条件" />
        </el-form-item>
        <el-form-item label="测试结果" prop="testResult">
          <el-input v-model="form.testResult" placeholder="请输入测试结果" />
        </el-form-item>
        <el-form-item label="备注" prop="remark">
          <el-input v-model="form.remark" placeholder="请输入备注" />
        </el-form-item>
      </el-form>
      <template #footer>
        <div class="dialog-footer">
          <el-button :loading="buttonLoading" type="primary" @click="submitForm">确 定</el-button>
          <el-button @click="cancel">取 消</el-button>
        </div>
      </template>
    </el-dialog>
  </div>
</template>

<script setup name="TestResult" lang="ts">
import { listTestResultAll, getTestResult, delTestResult, addTestResult, updateTestResult, listTestItem } from '@/api/qms/testResult';
import { TestResultVO, TestResultQuery, TestResultForm } from '@/api/qms/testResult/types';
import * as echarts from 'echarts';
import { listProdModels} from '@/api/qms/batch';

const { proxy } = getCurrentInstance() as ComponentInternalInstance;

const testResultList = ref<TestResultVO[]>([]);
const buttonLoading = ref(false);
const loading = ref(true);
const showSearch = ref(true);
const ids = ref<Array<string | number>>([]);
const single = ref(true);
const multiple = ref(true);
const total = ref(0);

// 设置默认时间为最近一个月
const now = new Date();
const dateRangeCreateTime = ref<[DateModelType, DateModelType]>([]);
const oneMonthAgo = new Date(now.getFullYear(), now.getMonth() - 1, now.getDate());
dateRangeCreateTime.value = [oneMonthAgo.toISOString().split('T')[0], now.toISOString().split('T')[0]] as [DateModelType, DateModelType];

const queryFormRef = ref<ElFormInstance>();
const testResultFormRef = ref<ElFormInstance>();
const chartRef = ref<HTMLDivElement | null>(null); // 新增 chartRef 引用

const testItemsOptions = ref<{ label: string, value: string }[]>([]); // 新增测试项目选项
const selectedTestItems = ref<string | string[]>([]); // 修改为兼容单选的类型

const dialog = reactive<DialogOption>({
  visible: false,
  title: ''
});

const initFormData: TestResultForm = {
  id: undefined,
  batchCode: undefined,
  testNum: undefined,
  testItem: undefined,
  voltage: undefined,
  loadCurrent: undefined,
  stdDistance: undefined,
  inductor: undefined,
  output: undefined,
  testValue: undefined,
  judgeDetail: undefined,
  testResult: undefined,
  remark: undefined
}
const prodModelOptions = ref<{ label: string, value: string }[]>([]);

const data = reactive<PageData<TestResultForm, TestResultQuery>>({
  form: {...initFormData},
  queryParams: {
    batchCode: undefined,
    testNum: undefined,
    testItem: undefined,
    voltage: undefined,
    loadCurrent: undefined,
    stdDistance: undefined,
    inductor: undefined,
    output: undefined,
    testValue: undefined,
    judgeDetail: undefined,
    testResult: undefined,
    startTime: undefined, // 新增 startTime 字段
    endTime: undefined, // 新增 endTime 字段
    prodModel: undefined, // 新增产品型号字段
    params: {
      createTime: undefined,
    }
  },
  rules: {
    id: [
      { required: true, message: "不能为空", trigger: "blur" }
    ],
    batchCode: [
      { required: true, message: "批次号不能为空", trigger: "blur" }
    ],
    testNum: [
      { required: true, message: "测试序号不能为空", trigger: "blur" }
    ],
    testItem: [
      { required: true, message: "测试项目不能为空", trigger: "blur" }
    ],
    voltage: [
      { required: true, message: "实际电压不能为空", trigger: "blur" }
    ],
    loadCurrent: [
      { required: true, message: "实际电流不能为空", trigger: "blur" }
    ],
    stdDistance: [
      { required: true, message: "标准距离不能为空", trigger: "blur" }
    ],
    inductor: [
      { required: true, message: "感应物不能为空", trigger: "blur" }
    ],
    output: [
      { required: true, message: "输出引脚不能为空", trigger: "blur" }
    ],
    testValue: [
      { required: true, message: "测试数据不能为空", trigger: "blur" }
    ],
    judgeDetail: [
      { required: true, message: "判断条件不能为空", trigger: "blur" }
    ],
    testResult: [
      { required: true, message: "测试结果不能为空", trigger: "blur" }
    ],
    prodModel: [
      { required: true, message: "产品型号不能为空", trigger: "blur" }
    ],
  }
});

const { queryParams, form, rules } = toRefs(data);

// 获取所有测试项目
const fetchTestItems = async () => {
  const res = await listTestItem();
  testItemsOptions.value = res.data.map(item => ({ label: item, value: item }));
}
// 获取所有产品型号
const fetchProdModels = async () => {
  const res = await listProdModels();
  prodModelOptions.value = res.data.map(item => ({ label: item, value: item }));
}

/** 查询测试结果列表 */
const getList = async () => {
  loading.value = true;
  queryParams.value.params = {};
  // 处理 selectedTestItems 的值，兼容单选和多选
  if (Array.isArray(selectedTestItems.value)) {
    queryParams.value.testItem = selectedTestItems.value.join(','); // 多选模式
  } else {
    queryParams.value.testItem = selectedTestItems.value; // 单选模式
  }
  proxy?.addDateRange(queryParams.value, dateRangeCreateTime.value, 'CreateTime');
  const res = await listTestResultAll(queryParams.value);
  testResultList.value = res.data;
  loading.value = false;
  updateChart(); // 更新图表
}

// 监听 testResultList 的变化，动态更新图表
watch(testResultList, () => {
  updateChart();
});

/** 取消按钮 */
const cancel = () => {
  reset();
  dialog.visible = false;
}

/** 表单重置 */
const reset = () => {
  form.value = {...initFormData};

  testResultFormRef.value?.resetFields();
}

/** 搜索按钮操作 */
const handleQuery = () => {
  // 校验时间范围是否超过三个月
  if (dateRangeCreateTime.value[0] && dateRangeCreateTime.value[1]) {
    const startDate = new Date(dateRangeCreateTime.value[0]);
    const endDate = new Date(dateRangeCreateTime.value[1]);
    const timeDiff = endDate.getTime() - startDate.getTime();
    const maxDuration = 90 * 24 * 60 * 60 * 1000; // 三个月的毫秒数

    if (timeDiff > maxDuration) {
      proxy?.$modal.msgWarning("时间范围不能超过三个月，请重新选择！");
      return;
    }
  }

  // 校验产品型号是否选择
  if (!queryParams.value.prodModel) {
    proxy?.$modal.msgWarning("请选择产品型号！");
    return;
  }

  // 检查测试项目是否选择
  if (!selectedTestItems.value.length) {
    proxy?.$modal.msgWarning("请选择测试项目！");
    return;
  }

  getList();
}

/** 重置按钮操作 */
const resetQuery = () => {
  dateRangeCreateTime.value = ['', ''];
  queryFormRef.value?.resetFields();
  selectedTestItems.value = []; // 重置选中的测试项目
  reset(); // 调用重置方法以清空时间段字段
}

/** 多选框选中数据 */
const handleSelectionChange = (selection: TestResultVO[]) => {
  ids.value = selection.map(item => item.id);
  single.value = selection.length != 1;
  multiple.value = !selection.length;
}

/** 新增按钮操作 */
const handleAdd = () => {
  reset();
  dialog.visible = true;
  dialog.title = "添加测试结果";
}

/** 修改按钮操作 */
const handleUpdate = async (row?: TestResultVO) => {
  reset();
  const _id = row?.id || ids.value[0]
  const res = await getTestResult(_id);
  Object.assign(form.value, res.data);
  dialog.visible = true;
  dialog.title = "修改测试结果";
}

/** 提交按钮 */
const submitForm = () => {
  testResultFormRef.value?.validate(async (valid: boolean) => {
    if (valid) {
      buttonLoading.value = true;
      if (form.value.id) {
        await updateTestResult(form.value).finally(() =>  buttonLoading.value = false);
      } else {
        await addTestResult(form.value).finally(() =>  buttonLoading.value = false);
      }
      proxy?.$modal.msgSuccess("操作成功");
      dialog.visible = false;
      await handleQuery();
    }
  });
}

/** 删除按钮操作 */
const handleDelete = async (row?: TestResultVO) => {
  const _ids = row?.id || ids.value;
  await proxy?.$modal.confirm('是否确认删除测试结果编号为"' + _ids + '"的数据项？').finally(() => loading.value = false);
  await delTestResult(_ids);
  proxy?.$modal.msgSuccess("删除成功");
  await handleQuery();
}

/** 导出按钮操作 */
const handleExport = () => {
  proxy?.download('qms/testResult/export', {
    ...queryParams.value
  }, `testResult_${new Date().getTime()}.xlsx`)
}

// 更新图表
const updateChart = () => {
  if (!chartRef.value) return;
  // 销毁已有的图表实例
  const existingChart = echarts.getInstanceByDom(chartRef.value);
  if (existingChart) {
    existingChart.dispose();
  }
  const chart = echarts.init(chartRef.value);

  // 定义20个不重复的颜色池
  const colorPool = [
    '#5470c6', '#91cc75', '#fac858', '#ee6666', '#73c0de',
    '#3ba272', '#fc8452', '#9a60b4', '#ea7ccc', '#5e7ce0',
    '#f0c757', '#ff9f7f', '#d48265', '#7fb80e', '#b37feb',
    '#4dc9b0', '#f47920', '#65b581', '#f08585', '#4a90e2',
    // 新增颜色以提高对比度
    '#ff6f61', '#ffd700', '#00ced1', '#da70d6', '#adff2f'
  ];

 // 使用Map替代Object更高效
const seriesMap = new Map<string, [string, number][]>();
testResultList.value.forEach((item) => {
  const key = item.testItem;
  const xValue = `${item.batchCode}-${item.testNum}`;
  if (!seriesMap.has(key)) {
    seriesMap.set(key, []);
  }

  const seriesArray = seriesMap.get(key)!;

  // 移除已有相同 xValue 的条目
  const filtered = seriesArray.filter(entry => entry[0] !== xValue);
  filtered.push([xValue, item.testValue]);

  seriesMap.set(key, filtered);
});


// 对每个 testItem 下的数据进行统一排序：先按 batchCode 再按 testNum
seriesMap.forEach((value, key) => {
  value.sort((a, b) => {
    const [batchA, testNumA] = a[0].split('-');
    const [batchB, testNumB] = b[0].split('-');

    // 先比较 batchCode（字符串比较）
    if (batchA !== batchB) {
      return batchA.localeCompare(batchB);
    }

    // 再比较 testNum（转换为数字比较）
    return Number(testNumA) - Number(testNumB);
  });

  seriesMap.set(key, value);
});


  // 解析最后一条数据的判断条件字段
  let upperLimit: number | null = null;
  let lowerLimit: number | null = null;
  if (testResultList.value.length > 0) {
    const lastItem = testResultList.value[testResultList.value.length - 1];
    const judgeDetail = lastItem.judgeDetail;
    console.log('judgeDetail:', judgeDetail)
    const regex = /(\d+\.\d+)\s*(V|mA|uA)\s*≤\s*result\s*≤\s*(\d+\.\d+)\s*(V|mA|uA)/;
    const match = judgeDetail.match(regex);
    console.log('match:', match)
    if (match) {
      lowerLimit = parseFloat(match[1]);
      upperLimit = parseFloat(match[3]);
    }
  }

  console.log('upperLimit:', upperLimit, 'lowerLimit:', lowerLimit)
  // 计算新的上下限值，增加20%
  const paddingPercentage = 0.1; // 20%的padding
  // 计算上限的20%是多少
  let padding = lowerLimit * paddingPercentage;

  console.log("padding",padding)

  let newLowerLimit = lowerLimit !== null ? (lowerLimit - padding) : undefined;
  let newUpperLimit = upperLimit !== null ? (upperLimit + padding) : undefined;


  const option = {
  tooltip: {
    trigger: 'axis', // 按坐标轴触发，适用于折线图/柱状图等
    formatter: (items: any[]) => {
      const result: string[] = [];
      items.forEach(item => {
        result.push(`${item.marker} ${item.seriesName}<br/>`);
        result.push(`序号：${item.name}<br/>`);
        result.push(`测试数据：${item.value[1]}<br/>`);
      });
      return result.join('');
    },
    backgroundColor: 'rgba(255, 255, 255, 0.9)',
    borderColor: '#ccc',
    textStyle: {
      color: '#333'
    }
  },
  grid: {
    top: '15%',
    left: '5%',
    right: '5%',
    containLabel: true
  },
  xAxis: {
    type: 'category',
    name: '序号',
    data: Array.from(seriesMap.values())[0]?.map(item => item[0]) || [] // 使用排序后的 x 值
  },
  yAxis: {
    type: 'value',
    name: '测试数据',
    min: newLowerLimit,
    max: newUpperLimit,
    axisLabel: {
      formatter: function (value: number) {
        return value;
      }
    }
  },
  legend: {
    data: Array.from(seriesMap.keys())
  },
  series: Array.from(seriesMap).map(([name, data], index) => ({
    name: name,
    data: data,
    type: 'line',
    animationDuration: 1000,
    lineStyle: {
      color: colorPool[index % colorPool.length],
      width: 2
    },
    itemStyle: {
      color: colorPool[index % colorPool.length],
      opacity: 0.8
    },
    markLine: {
      symbol: ['none', 'none'],
      label: {
        position: 'end',
        align: 'center',
        formatter: function (params: any) {
          const name = params.name || '';
          const value = params.value || '';
          return `${name}\n${value}`;
        }
      },
      lineStyle: {
        color: 'red',
        type: 'dashed'
      },
      data: [
        {
          name: '下限',
          yAxis: lowerLimit !== null ? lowerLimit : undefined
        },
        {
          name: '上限',
          yAxis: upperLimit !== null ? upperLimit : undefined
        }
      ]
    }
  })),
  dataZoom: [
    {
      type: 'slider',
      xAxisIndex: 0,
      start: 0,
      end: 100
    },
    {
      type: 'slider',
      yAxisIndex: 0,
      start: 0,
      end: 100
    },
    {
      type: 'inside',
      xAxisIndex: 0,
      start: 0,
      end: 100
    },
    {
      type: 'inside',
      yAxisIndex: 0,
      start: 0,
      end: 100
    }
  ]
};


  chart.setOption(option, true);
  window.addEventListener('resize', () => {
    chart.resize();
  });
}

onMounted(() => {
  fetchTestItems(); // 调用获取测试项目接口
  fetchProdModels(); // 调用获取产品型号接口
});
</script>
